﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Win32;

using Pixoneer.NXDL;            // 기본 기능 관련
using Pixoneer.NXDL.NGR;        // 그래픽 기능 관련
using Pixoneer.NXDL.NXMilmap;   // 군사지도 기능 관련

namespace XDL_MilmapView1
{
    /// <summary>
    /// MainWindow.xaml에 대한 상호 작용 논리
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            // XMilmapConfig.xml을 이용하여 군사지도를 위한 NXMilmapView 및 NXMilmapEngine을 초기화한다.
            if (!NXMilmapView.m_MapEngine.InitFromXML("c:\\Pixoneer\\Xdl1.2\\Config\\XMilmapConfig.xml"))
            {
                return;
            }

            // 마우스 휠을 끌고 당기는 것을 확대/축소 요인으로 계산하여 화면을 확대/축소한다.
            // 이 때, 계산된 공간 해상도와 가까운 축척(scale)이 있는 경우 자동적으로 해당 축척으로 바뀌면서 도시된다.
            // 마우스 휠을 끌고 당기는 것으로 화면의 축척을 변경하려면, NXMilmapView.eWheelZoomAction.ByScaleIndex으로 설정
            nxMilmapView1.WheelZoomAction = NXMilmapView.eWheelZoomAction.ByZoomFactor;

            nxMilmapView1.SetGeoToCenter(0, new XVertex2d(127.0, 36.0));

            // XMilmapConfig.xml에 설정된 지도 중 유효한 데이터만을 사용한다.
            // XMilmapConfig.xml에서 로딩한 스케일의 이름을 Scale을 위한 ComboBox에 추가한다.
            int numScale = NXMilmapView.m_MapEngine.GetNumofScale();
            for (int i = 0; i < numScale; i++)
            {
                string scaleName = NXMilmapView.m_MapEngine.GetScaleName(i);
                comboBoxScale.Items.Add(scaleName);
            }
            // 첫 번째 스케일을 기본값으로 하여 도시한다.
            if (numScale > 0)
                comboBoxScale.SelectedIndex = 0;

            // Mouse 이벤트 속성
            checkBoxInverseMouseButton.IsChecked = nxMilmapView1.InverseMouseButton;
            checkBoxInverseMouseWheel.IsChecked = nxMilmapView1.InverseMouseWheel;

            // Grid
            if (nxMilmapView1.GridType == NXMilmapView.eGridType.GridNone)
                comboBoxGrid.SelectedIndex = 0;
            else if (nxMilmapView1.GridType == NXMilmapView.eGridType.GridDegrees)
                comboBoxGrid.SelectedIndex = 1;
            else if (nxMilmapView1.GridType == NXMilmapView.eGridType.GridGARS)
                comboBoxGrid.SelectedIndex = 2;
            else
                comboBoxGrid.SelectedIndex = 0;

            // MilmapView 회전 설정
            NXMilmapDrawArgs drawArgs = nxMilmapView1.GetDrawArgs();
            if (drawArgs.Rotatable)
            {
                radioButtonRotateNone.IsChecked = false;
                radioButtonRotateMouse.IsChecked = true;
                radioButtonRotateUserInput.IsChecked = false;
            }
            else
            {
                radioButtonRotateNone.IsChecked = true;
                radioButtonRotateMouse.IsChecked = false;
                radioButtonRotateUserInput.IsChecked = false;
            }

            if (nxMilmapView1.ZoomType == NXMilmapView.eZoomType.MousePostion)
            {
                radioButtonZoomMousePosition.IsChecked = true;
                radioButtonZoomViewCenter.IsChecked = false;
            }
            // NXMilmapView.eZoomType.ViewCenter이 경우
            else
            {
                radioButtonZoomMousePosition.IsChecked = false;
                radioButtonZoomViewCenter.IsChecked = true;
            }

            // 지명 도시 여부 설정
            checkBoxShowPBP.IsChecked = nxMilmapView1.ShowPlaceName;
            // NXMilmapView는 기본적으로 화면 중앙에 붉은 십자를 표시힌다.
            checkBoxShowCross.IsChecked = true;
        }

        private void Window_Closed(object sender, EventArgs e)
        {
            Xfn.Close();
        }

        private void checkBoxInverseMouseButton_Checked(object sender, RoutedEventArgs e)
        {
            // MilmapView는 기본적으로 마우스 왼쪽 버튼으로 화면 이동을, 오른쪽 버튼으로 화면 회전을 한다.
            // 마우스 왼쪽 버튼과 오른쪽 버튼에 대한 기능을 전환하려면, NXMilmapView의 InverseMouseButton을 true로 설정하면 된다.
            nxMilmapView1.InverseMouseButton = true;
        }

        private void checkBoxInverseMouseButton_Unchecked(object sender, RoutedEventArgs e)
        {
            // MilmapView는 기본적으로 마우스 왼쪽 버튼으로 화면 이동을, 오른쪽 버튼으로 화면 회전을 한다.
            // 마우스 왼쪽 버튼과 오른쪽 버튼에 대한 기능을 전환 후 해제하려면, NXMilmapView의 InverseMouseButton을 false로 설정하면 된다.
            nxMilmapView1.InverseMouseButton = false;
        }

        private void checkBoxInverseMouseWheel_Checked(object sender, RoutedEventArgs e)
        {
            // MilmapView의 WheelZoomAction 값이 NXMilmapView.eWheelZoomAction.ByZoomFactor인 경우 
            // MilmapView는 기본적으로 마우스 휠을 당기면 화면확대가, 밀면 화면축소가 된다.
            // 이를 전환하려면 MilmapView의 InverseMouseWheel을 true로 설정하면 된다.
            nxMilmapView1.InverseMouseWheel = true;
        }

        private void checkBoxInverseMouseWheel_Unchecked(object sender, RoutedEventArgs e)
        {
            // MilmapView의 WheelZoomAction 값이 NXMilmapView.eWheelZoomAction.ByZoomFactor인 경우 
            // MilmapView는 기본적으로 마우스 휠을 당기면 화면확대가, 밀면 화면축소가 된다.
            // 이를 전환 후 해제하려면 MilmapView의 InverseMouseWheel을 false로 설정하면 된다.
            nxMilmapView1.InverseMouseWheel = false;
        }

        private void comboBoxScale_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // 축척(scale)을 선택하지 않은 경우에는 축척 변경을 하지 않는다.
            Nullable<bool> result = checkBoxScale.IsChecked;
            if (result != true) return;

            int numScale = NXMilmapView.m_MapEngine.GetNumofScale();
            int idx = comboBoxScale.SelectedIndex;

            // 축척 인덱스의 유효함 체크
            if (idx < 0 || idx >= numScale) return;

            NXMilmapDrawArgs drawArgs = nxMilmapView1.GetDrawArgs();

            // 현재 화면 중심으로 해서 변경된 축척 인덱스로 변경
            nxMilmapView1.SetGeoToCenter(idx, drawArgs.CenterPos);

            // 화면 갱신 요청
            nxMilmapView1.RefreshScreen();
        }

        private void buttonCenterMove_Click(object sender, RoutedEventArgs e)
        {
            double posLon = Double.Parse(textBoxCenterLon.Text);
            double posLat = Double.Parse(textBoxCenterLat.Text);

            // 입력 화면 중심 위경도 좌표 유효 체크
            if (posLon < -180.0 || posLon > 180.0) return;
            if (posLat < -90.0 || posLat > 90.0) return;

            NXMilmapDrawArgs drawArgs = nxMilmapView1.GetDrawArgs();
            // 현재 설정된 축척을 기반으로 화면 중심을 이동한다.
            nxMilmapView1.SetGeoToCenter(drawArgs.ScaleIndex, new XVertex2d(posLon, posLat));

            // 화면 갱신 요청
            nxMilmapView1.RefreshScreen();
        }

        private void comboBoxGrid_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            int gridIndex = comboBoxGrid.SelectedIndex;
            if (gridIndex == 0) // none
            {
                nxMilmapView1.GridType = NXMilmapView.eGridType.GridNone;
                nxMilmapView1.ShowGrid = false;
            }

            else if (gridIndex == 1)    // Degrees
            {
                nxMilmapView1.GridType = NXMilmapView.eGridType.GridDegrees;
                nxMilmapView1.ShowGrid = true;
            }

            else if (gridIndex == 2)    // GARS
            {
                nxMilmapView1.GridType = NXMilmapView.eGridType.GridGARS;
                nxMilmapView1.ShowGrid = true;
            }

            // 화면 갱신 요청
            nxMilmapView1.RefreshScreen();
        }

        // 화면이 회전되지 않도록 설정
        private void radioButtonRotateNone_Checked(object sender, RoutedEventArgs e)
        {
            NXMilmapDrawArgs drawArgs = nxMilmapView1.GetDrawArgs();
            drawArgs.Rotatable = false;
        }

        // 화면 회전이 가능하도록 설정
        private void radioButtonRotateMouse_Checked(object sender, RoutedEventArgs e)
        {
            NXMilmapDrawArgs drawArgs = nxMilmapView1.GetDrawArgs();
            drawArgs.Rotatable = true;
        }

        // 화면 회전이 가능하도록 설정하고 사용자 입력하는 각도에 따라 화면 회전
        private void textBoxRotateUserInput_TextChanged(object sender, TextChangedEventArgs e)
        {
            // 사용자 입력을 선택하지 않은 경우 반영하지 않는다.
            Nullable<bool> result = radioButtonRotateUserInput.IsChecked;
            if (result != true) return;

            NXMilmapDrawArgs drawArgs = nxMilmapView1.GetDrawArgs();
            double angleRotate = Double.Parse(textBoxRotateUserInput.Text);

            // 화면 회전 각도 설정
            drawArgs.AzimuthAng = XAngle.FromDegree(angleRotate);
            drawArgs.Rotatable = true;
            nxMilmapView1.RefreshScreen();
        }

        // 화면 확대/축소의 중심을 화면 중심으로 설정
        private void radioButtonZoomViewCenter_Checked(object sender, RoutedEventArgs e)
        {
            nxMilmapView1.ZoomType = NXMilmapView.eZoomType.ViewCenter;
        }

        // 화면 확대/축소 중심을 마우스 위치 중심으로 설정
        private void radioButtonZoomMousePosition_Checked(object sender, RoutedEventArgs e)
        {
            nxMilmapView1.ZoomType = NXMilmapView.eZoomType.MousePostion;
        }

        // 지명 도시 여부를 설정
        private void checkBoxShowPBP_Checked(object sender, RoutedEventArgs e)
        {
            nxMilmapView1.ShowPlaceName = true;
            nxMilmapView1.RefreshScreen();
        }

        private void checkBoxShowPBP_Unchecked(object sender, RoutedEventArgs e)
        {
            nxMilmapView1.ShowPlaceName = false;
            nxMilmapView1.RefreshScreen();
        }

        // 화면 중심에 십자 표시 여부 설정
        private void checkBoxShowCross_Checked(object sender, RoutedEventArgs e)
        {
            bool result = (bool)checkBoxShowCross.IsChecked;
            nxMilmapView1.ShowCross(result);
            nxMilmapView1.RefreshScreen();
        }

        private void checkBoxShowCross_Unchecked(object sender, RoutedEventArgs e)
        {
            bool result = (bool)checkBoxShowCross.IsChecked;
            nxMilmapView1.ShowCross(result);
            nxMilmapView1.RefreshScreen();
        }
    }  
}
